// This file was generated by lezer-generator. You probably shouldn't edit it.
export const
  printKeyword = 1,
  indent = 177,
  dedent = 178,
  newline = 179,
  newlineBracketed = 180,
  newlineEmpty = 181,
  eof = 182,
  Comment = 2,
  Script = 3,
  BinaryExpression = 6,
  ParenthesizedExpression = 21,
  ParenL = 22,
  binaryTest = 23,
  CompareOp = 26,
  unaryTest = 30,
  lambdaParams = 36,
  VariableName = 37,
  YieldExpression = 43,
  TupleExpression = 47,
  ComprehensionExpression = 48,
  ArrayExpression = 52,
  BracketL = 53,
  ArrayComprehensionExpression = 55,
  DictionaryExpression = 56,
  BraceL = 57,
  DictionaryComprehensionExpression = 59,
  SetExpression = 60,
  SetComprehensionExpression = 61,
  ArgList = 63,
  subscript = 218,
  PropertyName = 67,
  Number = 68,
  String = 69,
  FormatString = 70,
  FormatReplacement = 71,
  FormatConversion = 72,
  TypeDef = 78,
  UpdateOp = 81,
  ImportStatement = 97,
  importList = 243,
  CNextStarter = 106,
  CNextPlotExpression = 107,
  CNextDataFrameExpresion = 108,
  CNextPlotYDimExpression = 109,
  CNextYDimColumnNameExpression = 110,
  CNextPlotAddDimKeyword = 111,
  CNextPlotXDimExpression = 115,
  CNextXDimColumnNameExpression = 116,
  IfStatement = 120,
  Body = 121,
  TryStatement = 126,
  FunctionDefinition = 132,
  ParamList = 134,
  ClassDefinition = 137,
  Decorator = 140,
  At = 141
