// This file was generated by lezer-generator. You probably shouldn't edit it.
import {LRParser} from "@lezer/lr"
import {NodeProp} from "@lezer/common"
const spec_identifier = {__proto__:null,"|":20, "&":22, not:24, index:28, isin:32, True:42, False:42, isna:44, notna:46}
export const parser = LRParser.deserialize({
  version: 13,
  states: "(^OVQPOOO[QPO'#C_OOQO'#Cw'#CwOpQPO'#C^QOQPOOOxQPO'#CbOOQO'#Ci'#CiO!TQPO'#DQObQPO'#DTOOQO'#C}'#C}O!cQPO'#CeOOQO'#Ca'#CaO!qQPO'#CvO!yQPO,58yO!|QPO,58yObQPO'#C}OOQO-E6u-E6uO#RQPO,58|O#WQPO,59lOOQO,59l,59lO#iQPO,59oObQPO,59jObQPO,59jO#tQPO'#DUOOQO,59b,59bO#|QPO,59bOOQO1G.e1G.eO$RQPO1G.eO$WQPO,59iOOQO1G.h1G.hOOQO'#Cn'#CnOOQO'#Cm'#CmO$iQPO'#CmOOQO1G/W1G/WOOQO1G/Z1G/ZO$tQPO1G/UOOQO1G/U1G/UO%VQPO,59pO%_QPO,59pOOQO1G.|1G.|O%gQPO7+$PO%lQPO'#DSOOQO,59X,59XO%tQPO,59XOOQO,59e,59eO%yQPO1G/[OOQO-E6w-E6wOOQO<<Gk<<GkO&RQPO,59nO&^QPO,59nOOQO1G.s1G.sPkQPO'#CyOOQO,59d,59dO&fQPO1G/YOOQO-E6v-E6vP&qQPO'#Cx",
  stateData: "&y~OpOS~OSPO~OS[OVTO[_O^VOhWOuUO~OSPOnQX~OWaOcUXdUX~O_bO`bOfcOgcO~OYeOZfOcXXdXX~OdhOuUO~OckOdjO~OVmO~OSpOVnOeoOsoOunO~OYeOZfOirO~OcuOdxX~OdwO~OS[O~OYeOZfOcqadqaiqa~OVnOdzOunO~OZfOYricridriiri~OuUOdxa~Oc}Odxa~Od!PO~Oc!QOdvX~Od!SO~OuUOdxi~OVnOunOdva~Oc!VOdva~OVnOunOdvi~OVnOunO~O",
  goto: "#|yPPz}P!R!UPP!UPPP!XPPP!h!kPPPPPPP!v!|#S#YPPP#`#oP#oP#v#o#yRSOTQORR]PRZPYVPW_efQg[V|u}!TRqbQobQypV!U!Q!V!XQ^PRxkQROR`RQ!RyR!W!RQvgR!OvQYPQdWQl_QseRtfZXPW_efR{pRi[",
  nodeNames: "⚠ Script QueryStatement SimpleQueryExpression [ IndexExpression IndexSlicingExpression Number : BooleanIndexExpression | & not ColumnNameExpression index CompareOp isin IndexSelectorExpression ColumnValueExpression , ] Boolean isna notna ( ) ColumnFilterExpression",
  maxTerm: 40,
  nodeProps: [
    [NodeProp.group, 2,"Statement"]
  ],
  skippedNodes: [0],
  repeatNodeCount: 3,
  tokenData: ".S~RsXY#`[]#`pq#`qr#zrs$Vwx$|xy%nyz%s|}%x!O!P%}!Q!R'f!R![(d![!]*p!^!_*u!_!`#z!`!a+Q!c!d+Y!d!e+n!e!t+Y!t!u-Q!u!w+Y!w!x,f!x!}+Y!}#O-x#O#P#q#P#Q-}#R#S+Y#T#U+Y#U#V+n#V#f+Y#f#g-Q#g#i+Y#i#j,f#j#o+Y$g~+Y~#eSp~XY#`[]#`pq#`#O#P#q~#tQYZ#`]^#`~#}P!_!`$Q~$VO_~~$[Vu~OY$VZ]$V^r$Vrs$qs#O$V#O#P$v#P~$V~$vOu~~$yPO~$V~%RVu~OY$|Z]$|^w$|wx$qx#O$|#O#P%h#P~$|~%kPO~$|~%sOh~~%xOi~~%}Oc~~&QP!Q![&T~&YUV~!Q![&T!g!h&l!l!m'a#R#S&T#X#Y&l#^#_'a~&oR{|&x}!O&x!Q!['O~&{P!Q!['O~'TSV~!Q!['O!l!m'a#R#S'O#^#_'a~'fOV~~'k]V~!O!P%}!Q![(d!d!e)O!g!h&l!l!m'a!q!r)j!z!{*O#R#S(d#U#V)O#X#Y&l#^#_'a#c#d)j#l#m*O~(iVV~!O!P%}!Q![(d!g!h&l!l!m'a#R#S(d#X#Y&l#^#_'a~)RR!Q!R)[!R!S)[#R#S)[~)aRV~!Q!R)[!R!S)[#R#S)[~)mQ!Q!Y)s#R#S)s~)xQV~!Q!Y)s#R#S)s~*RS!Q![*_!c!i*_#R#S*_#T#Z*_~*dSV~!Q![*_!c!i*_#R#S*_#T#Z*_~*uOW~~*zQ_~!_!`$Q!`!a$Q~+VP_~!_!`$Q~+_Ts~!Q![+Y!c!}+Y#R#S+Y#T#o+Y$g~+Y~+sZs~rs$Vwx$|!Q![+Y!c!t+Y!t!u,f!u!}+Y#R#S+Y#T#f+Y#f#g,f#g#o+Y$g~+Y~,kVs~rs$Vwx$|!Q![+Y!c!}+Y#R#S+Y#T#o+Y$g~+Y~-VZs~rs$Vwx$|!Q![+Y!c!t+Y!t!u,f!u!}+Y#R#S+Y#T#U+Y#U#V,f#V#o+Y$g~+Y~-}OS~~.SOd~",
  tokenizers: [0],
  topRules: {"Script":[0,1]},
  specialized: [{term: 35, get: value => spec_identifier[value] || -1}],
  tokenPrec: 0
})
