// store all ids for automate testing
const CypressIds = {
    codeEditor: "code-editor",
    dfEditor: "df-editor",
    projectRoot: "project-root",
    newFileItem: "new-file-item",
    dfStatsCheckboxAll: "df-stats-checkbox",
    dfViewMode: "df-viewmode",
    dfSummaryTable: "df-summary-table",
};

export default CypressIds;
